import React, { Fragment, useEffect } from 'react';
import {Button, Empty, Table, Space, Input} from 'antd';
import {CheckOutlined} from "@ant-design/icons";
import * as AntdIcon from '@ant-design/icons';
import FileView from '../../../Account/components/TablePack/components/FileView';
import TagPack from '../../../components/TagPack'

import styles from './index.less';

const DataTable = props => {
  const { value, onChange, schema } = props;
  const {
    disabled,
    fieldList,
    optionRender ,
    titleShow ,
    pagination ,
    scrollX,
    scrollY,
    statusOption
  } = schema;

  const hasHash = (url) => {
    return url.includes('#');
  }

  const doAction = (_, row, index , actionItem) => {
    const list = [...value];

    switch (actionItem.action) {
      case "delete":
        list.splice(index , 1);
        onChange(list);
        break;
      case "update":
        list.forEach((item , i) => {
          if(i === index){
            if(!row.isEditable){
              row.isEditable = true;
              row.type = row.type || "update";
            } else {
              delete row.isEditable
            }
          }
        });
        onChange(list);
        break;
      case "href":
        if(!row.isEditable){
          let str = actionItem.url;
          const reg_g = /\$\{(.+?)\}/g;
          const result = str.match(reg_g);

          const list = []
          const reg = /\$\{(.+?)\}/;
          for (let i = 0; i < result.length; i++) {
            const item = result[i]
            list.push(item.match(reg)[1])
          }

          list.forEach(l => {
            if(row[l]){
              const s = "${" + l + "}"
              str = str.replace(s, row[l])
            }
          })

          if(hasHash(actionItem.url)){
            window.history.pushState(null, null ,  str)
          } else {
            window.open(str)
          }
        }
        break;
      default:
        break
    }
  }

  const changeValue = (val, index, type) => {
    const list = [...value];

    list.forEach((item , i) => {
      if(i === index){
        item[type] = val;
      }
    })

    onChange(list);
  }

  // 操作栏渲染
  const getOptionRender = (_, row, index) => {
    const eleList = [];
    optionRender?.forEach((item , i) => {
      let Element;
      if (item.type === 'icon' && item.value) {
        const Icon = AntdIcon[item.value];
        Element = <Button size={'small'}
                          type="text"
                          key={i}
                          shape={'circle'}
                          onClick={() => doAction(_, row, index , item)}
                          icon={row.isEditable ? <CheckOutlined /> : <Icon />} />;
      } else if (item.type === 'text') {
        Element = <Button size={'small'}
                          key={i}
                          onClick={() => doAction(_, row, index, item)}
                          type="text">{row.isEditable ? "完成" : item.value}</Button>;
      } else {
        Element = <Button size={'small'}
                          key={i}
                          onClick={() => doAction(_, row, index, item)}
                          type="link">{row.isEditable ? "完成" : item.value}</Button>;
      }

      eleList.push(Element);
    });


    return <Fragment>
      <Space>
        {eleList}
      </Space>
    </Fragment>;
  };

  // 字段栏渲染
  const getFieldRender = (_, row, index , item) => {
    let ele;
    switch (item.type) {
      case "text":
        if(!item.isMapped){
          ele = <span style={{color: item.color}}>{_}</span>
        } else {
          if(item.parseStr){
            const json = JSON.parse(item.parseStr);
            ele = <span style={{color: json[_].color}}>{json[_].label}</span>
          } else {
            ele = <span>调试中...</span>
          }
        }
        break;
      case "tag":
        if(!item.isMapped){
          ele = <TagPack color={item.color} text={_} />
        } else {
          if(item.parseStr){
            const json = JSON.parse(item.parseStr);
            ele = <TagPack color={json[_].color} text={json[_].label} />
          } else {
            ele = <span>调试中...</span>
          }

        }
        break;
    }

    return ele
  }

  const getColumns = () => {
    const arr = [];
    const list = statusOption?.length !== 0 ? statusOption : fieldList;
    let isGIS = false;
    list?.forEach(item => {
      if(item.fieldType === "gis"){
        isGIS = true
      }
    })

    list?.forEach(item => {
      if (item.fieldType === '台账' && schema?.widget === 'FileUpload') {
        arr.push({
          title: item.fieldAliasName,
          dataIndex: !isGIS ? item.fieldName : `${item.fieldType}-${item.fieldName}`,
          key: !isGIS ? item.fieldName : `${item.fieldType}-${item.fieldName}`,
          width: item.width || 120,
          render: (_, row) => {
            return <FileView value={row[`${item.fieldType}-${item.fieldName}`]} {...schema} />;
          },
        });
      } else if (item.fieldType === '台账' && item.fieldName === '反馈状态') {
        //统一最后一行添加
      } else {
        const dataIndex =  !isGIS ? item.fieldName : `${item.fieldType}-${item.fieldName}`;

        arr.push({
          title: item.fieldAliasName,
          dataIndex,
          key: dataIndex,
          width: item.width || 120,
          render: (_, row , index) => {
            return (
              <Fragment>
                {
                  row.isEditable ? (
                    <Input value={_}
                           onChange={e => changeValue(e.target.value , index , dataIndex)} />
                  ) : getFieldRender(_, row, index , item)
                }
              </Fragment>
            )
          }
        });
      }
    });

    if (!disabled) {
      arr.push({
        title: '操作',
        dataIndex: '',
        key: '',
        width: 120,
        render: (_, row, index) => {
          return getOptionRender(_, row, index);
        },
      });
    }

    return arr
  };

  const getData = () => {
    window.relationForm = {
      configs: [
        {
          '台账名称': schema.accountName,
          '映射字段': [{ fromField: '工单编号', toField: '工单编号' }],
          widget: schema.widget,
          schema: schema.layerName,
          accountName: schema.accountName,
          tableName: schema.tableName
        }
      ],
      data: []
    }
  }

  const columns = getColumns();

  useEffect(() => {
    getData();
  }, []);

  return (
    <Fragment>
      <div className={styles.dataTable}>
        <div className={styles.tableTitle}>{titleShow}</div>
        {columns.length !== 0 ? (
          <Table
            size="small"
            rowKey={disabled ? 'gis-设备编号' : 'ID'}
            bordered
            columns={columns}
            pagination={pagination}
            scroll={!pagination && {
              x: scrollX,
              y: scrollY
            }}
            dataSource={value}
          />
        ) : (
          <Empty />
        )}
      </div>
    </Fragment>
  );
};

export default DataTable;
